@@ -9,10 +9,10 @@
  * Copyright:
  * Copyright (C) 2010 Greenbone Networks GmbH
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2,
- * or, at your option, any later version as published by the Free
- * Software Foundation
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,7 +38,8 @@
 
 #include <pwd.h>
 #include <unistd.h>
-
+#include <sys/types.h>
+#include <grp.h>
 
 /**
  * @brief Sets an error and return \param errorcode.
@@ -89,14 +90,21 @@ drop_privileges (gchar * username, GError ** error)
     {
       if ((user_pw = getpwnam (username)))
         {
+          if (initgroups (username, user_pw->pw_gid) != 0)
+            return drop_privileges_error
+                    (error,
+                     OPENVAS_DROP_PRIVILEGES_FAIL_SUPPLEMENTARY,
+                     "Failed to drop supplementary groups privileges!\n");
           if (setgid (user_pw->pw_gid) != 0)
-            return drop_privileges_error (error,
-                                          OPENVAS_DROP_PRIVILEGES_FAIL_DROP_GID,
-                                          "Failed to drop group privileges!\n");
+            return drop_privileges_error
+                    (error,
+                     OPENVAS_DROP_PRIVILEGES_FAIL_DROP_GID,
+                     "Failed to drop group privileges!\n");
           if (setuid (user_pw->pw_uid) != 0)
-            return drop_privileges_error (error,
-                                          OPENVAS_DROP_PRIVILEGES_FAIL_DROP_UID,
-                                          "Failed to drop user privileges!\n");
+            return drop_privileges_error
+                    (error,
+                     OPENVAS_DROP_PRIVILEGES_FAIL_DROP_UID,
+                     "Failed to drop user privileges!\n");
         }
       else
         {
