@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,20 +13,23 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
+#define _GNU_SOURCE
+
 #include <stdlib.h>             /* for srand48 */
 #include <string.h>             /* for strlen */
 #include <unistd.h>             /* for getpid */
+#include <string.h>             /* for memmem */
 
 #include <glib.h>               /* for g_get_current_dir and others */
 #include <glib/gstdio.h>        /* for g_chdir */
 
-#include "system.h"             /* for efree */
-
-#include "nasl_regex.h"
+#include "regex.h"
+#include "../misc/openvas_logging.h"
+#include "../misc/prefs.h"           /* for prefs_get */
 
 #include "nasl.h"
 #include "nasl_tree.h"
@@ -35,10 +38,8 @@
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
 #include "exec.h"
-#include "preparse.h"
 
 #include "nasl_debug.h"
-#include "strutils.h"
 #include "nasl_init.h"
 
 #ifndef NASL_DEBUG
@@ -47,33 +48,15 @@
 
 extern int naslparse (naslctxt *);
 
-/**
- * @brief Check if a lexical context is authenticated.
- * 
- * Prints a nasl error if the context is not authenticated
- * 
- * @return 0 if the context is authenticated, -1 otherwise.
- */
-int
-check_authenticated (lex_ctxt * lexic)
-{
-  if (lexic->authenticated == 1)
-    return 0;
-  else
-    {
-      nasl_perror (lexic,
-                   "A non-authenticated script attempted to use an authenticated function - returning NULL\n");
-      return -1;
-    }
-}
-
-
 static int
 cell2bool (lex_ctxt * lexic, tree_cell * c)
 {
   tree_cell *c2;
   int flag;
 
+#if 0
+  nasl_dump_tree (c);
+#endif
 
   if (c == NULL || c == FAKE_CELL)
     return 0;
@@ -82,25 +65,10 @@ cell2bool (lex_ctxt * lexic, tree_cell * c)
     {
     case CONST_INT:
       return c->x.i_val != 0;
+
     case CONST_STR:
     case CONST_DATA:
-      if (c->size == 0)
-        return 0;
-      if (c->x.str_val[0] == '0' && c->size == 1)
-        {
-          /* 
-           * This gives the same semantics as Perl ("0" is false), 
-           * but I do not agree with it. 
-           * This piece of code is here from the begining of NASL2; it 
-           * probably fixed some compatibility issue with old 
-           * quick & dirty scripts. 
-           * I added this warning to check if we can switch to a
-           * simpler behaviour (empty string = false, not empty = true)
-           */
-          nasl_perror (lexic, "cell2boll: string '0' is FALSE\n");
-          return 0;
-        }
-      return 1;
+      return c->size != 0;
 
     case REF_ARRAY:
     case DYN_ARRAY:
@@ -115,20 +83,6 @@ cell2bool (lex_ctxt * lexic, tree_cell * c)
     }
 }
 
-static int
-cvt_bool (lex_ctxt * lexic, tree_cell * c)
-{
-  int flag;
-
-#if 0
-  nasl_dump_tree (c);
-#endif
-  flag = cell2bool (lexic, c);
-  /* free(c); free_tree(c); */
-  return flag;
-}
-
-
 static int
 cell2int3 (lex_ctxt * lexic, tree_cell * c, int warn)
 {
@@ -205,87 +159,32 @@ cell2str (lex_ctxt * lexic, tree_cell * c)
   switch (c->type)
     {
     case CONST_INT:
-      p = malloc (16);
-      if (p != NULL)
-        snprintf (p, 16, "%d", c->x.i_val);     /* RATS: ignore */
-      return p;
+      return g_strdup_printf ("%d", c->x.i_val);
 
     case CONST_STR:
     case CONST_DATA:
       if (c->x.str_val == NULL)
-        p = estrdup ("");
+        p = g_strdup ("");
       else
-        p = nasl_strndup (c->x.str_val, c->size);
+        p = g_memdup (c->x.str_val, c->size + 1);
       return p;
 
     case REF_ARRAY:
     case DYN_ARRAY:
       a = c->x.ref_val;
       p = (char *) array2str (a);
-      return estrdup (p);
+      return g_strdup (p);
 
     default:
       c2 = nasl_exec (lexic, c);
       p = cell2str (lexic, c2);
       deref_cell (c2);
       if (p == NULL)
-        p = estrdup ("");
+        p = g_strdup ("");
       return p;
     }
 }
 
-
-#ifdef DEPRECAT_CODE
-char *
-cell2str_and_size (lex_ctxt * lexic, tree_cell * c, int *sz)
-{
-  char *p;
-  tree_cell *c2;
-
-  if (c == NULL || c == FAKE_CELL)
-    {
-#if NASL_DEBUG > 0
-      nasl_perror (lexic, "Cannot convert NULL or FAKE cell to string\n");
-#endif
-      return NULL;
-    }
-
-  switch (c->type)
-    {
-    case CONST_INT:
-      p = malloc (16);
-      if (p == NULL)
-        return NULL;
-      snprintf (p, 16, "%d", c->x.i_val);       /* RATS: ignore */
-      if (sz != NULL)
-        *sz = strlen (p);
-      return p;
-
-    case CONST_STR:
-    case CONST_DATA:
-      if (c->x.str_val == NULL)
-        p = estrdup ("");
-      else
-        p = nasl_strndup (c->x.str_val, c->size);
-      if (sz != NULL)
-        *sz = c->size;
-      return p;
-
-    default:
-      c2 = nasl_exec (lexic, c);
-      p = cell2str_and_size (lexic, c2, sz);
-      deref_cell (c2);
-      if (p == NULL)
-        {
-          p = estrdup ("");
-          if (sz != NULL)
-            *sz = 0;
-        }
-      return p;
-    }
-}
-#endif
-
 /**
  * @return A 'referenced' cell.
  */
@@ -333,7 +232,7 @@ cell_cmp (lex_ctxt * lexic, tree_cell * c1, tree_cell * c2)
   c2 = cell2atom (lexic, c2);
 
   /*
-   * Comparing anything to something else which is entirely different 
+   * Comparing anything to something else which is entirely different
    * may lead to unpredictable results.
    * Here are the rules:
    * 1. No problem with same types, although we do not compare arrays yet
@@ -430,22 +329,22 @@ cell_cmp (lex_ctxt * lexic, tree_cell * c1, tree_cell * c2)
       if (flag == 0)
         flag = len_s1 - len_s2;
 
-      efree (&s1);
-      efree (&s2);
+      g_free (s1);
+      g_free (s2);
       deref_cell (c1);
       deref_cell (c2);
       return flag;
 
     case REF_ARRAY:
     case DYN_ARRAY:
-      fprintf (stderr, "cell_cmp: cannot compare arrays yet\n");
+      log_legacy_write ("cell_cmp: cannot compare arrays yet\n");
       deref_cell (c1);
       deref_cell (c2);
       return 0;
 
     default:
-      fprintf (stderr, "cell_cmp: don't known how to compare %s and %s\n",
-               nasl_type_name (typ1), nasl_type_name (typ2));
+      log_legacy_write ("cell_cmp: don't known how to compare %s and %s\n",
+                        nasl_type_name (typ1), nasl_type_name (typ2));
       deref_cell (c1);
       deref_cell (c2);
       return 0;
@@ -876,13 +775,14 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
   int len1, len2;
   nasl_func *pf = NULL;
   int i, n;
-  unsigned long sz;
 
 
 #if 0
   nasl_dump_tree (st);          /* See rt.value, rt.type, rt.length */
 #endif
 
+  if (st)
+    lexic->line_nb = st->line_nb;
   /* return */
   if (lexic->ret_val != NULL)
     {
@@ -916,7 +816,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
       if (ret == NULL)
         return NULL;
 #endif
-      if (cvt_bool (lexic, ret))
+      if (cell2bool (lexic, ret))
         ret2 = nasl_exec (lexic, st->link[1]);
       else if (st->link[2] != NULL)     /* else branch */
         ret2 = nasl_exec (lexic, st->link[2]);
@@ -957,7 +857,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
           /* condition */
           if ((ret = nasl_exec (lexic, st->link[1])) == NULL)
             return NULL;        /* We can return here, as NULL is false */
-          flag = cvt_bool (lexic, ret);
+          flag = cell2bool (lexic, ret);
           deref_cell (ret);
           if (!flag)
             break;
@@ -1001,7 +901,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
           /* Condition */
           if ((ret = nasl_exec (lexic, st->link[0])) == NULL)
             return NULL;        /* NULL is false */
-          flag = cvt_bool (lexic, ret);
+          flag = cell2bool (lexic, ret);
           deref_cell (ret);
           if (!flag)
             break;
@@ -1055,7 +955,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
           if (ret == NULL)
             return NULL;
 #endif
-          flag = cvt_bool (lexic, ret);
+          flag = cell2bool (lexic, ret);
           deref_cell (ret);
           if (flag)
             break;
@@ -1145,7 +1045,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
 #endif
 
       /*
-       * I wonder... 
+       * I wonder...
        * Will nasl_exec be really called with NODE_EXEC or NODE_ARG?
        */
     case NODE_DECL:            /* Used in function declarations */
@@ -1368,6 +1268,7 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
 #endif
       switch (flag)
         {
+        long sz;
         case CONST_INT:
           x = tc1->x.i_val;
           y = cell2int (lexic, tc2);
@@ -1394,13 +1295,13 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
             }
 
           sz = len1 + len2;
-          s3 = emalloc (sz);
+          s3 = g_malloc0 (sz + 1);
           if (len1 > 0)
             memcpy (s3, s1 != NULL ? s1 : tc1->x.str_val, len1);
           if (len2 > 0)
             memcpy (s3 + len1, s2 != NULL ? s2 : tc2->x.str_val, len2);
-          efree (&s1);
-          efree (&s2);
+          g_free (s1);
+          g_free (s2);
           ret = alloc_tree_cell (0, s3);
           ret->type = flag;
           ret->size = sz;
@@ -1498,9 +1399,9 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
             }
 
           if (len2 == 0 || len1 < len2
-              || (p = (char *) nasl_memmem (p1, len1, p2, len2)) == NULL)
+              || (p = memmem (p1, len1, p2, len2)) == NULL)
             {
-              s3 = emalloc (len1);
+              s3 = g_malloc0 (len1 + 1);
               memcpy (s3, p1, len1);
               ret = alloc_tree_cell (0, s3);
               ret->type = flag;
@@ -1508,15 +1409,15 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
             }
           else
             {
-              sz = len1 - len2;
+              long sz = len1 - len2;
               if (sz <= 0)
                 {
                   sz = 0;
-                  s3 = estrdup ("");
+                  s3 = g_strdup ("");
                 }
               else
                 {
-                  s3 = emalloc (sz);
+                  s3 = g_malloc0 (sz + 1);
                   if (p - p1 > 0)
                     memcpy (s3, p1, p - p1);
                   if (sz > p - p1)
@@ -1527,8 +1428,8 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
               ret->type = flag;
             }
 
-          efree (&s1);
-          efree (&s2);
+          g_free (s1);
+          g_free (s2);
           break;
 
         default:
@@ -1683,12 +1584,12 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
         }
 
       if (len1 <= len2)
-        flag = ((void *) nasl_memmem (p2, len2, p1, len1) != NULL);
+        flag = (memmem (p2, len2, p1, len1) != NULL);
       else
         flag = 0;
 
-      efree (&s1);
-      efree (&s2);
+      g_free (s1);
+      g_free (s2);
       deref_cell (tc1);
       deref_cell (tc2);
       if (st->type == COMP_MATCH)
@@ -1707,8 +1608,8 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
       s1 = cell2str (lexic, st->link[0]);
       if (s1 == NULL)
         return 0;
-      flag = nasl_regexec (st->x.ref_val, s1, 0, NULL, 0);
-      free (s1);
+      flag = regexec (st->x.ref_val, s1, 0, NULL, 0);
+      g_free (s1);
       if (st->type == COMP_RE_MATCH)
         return bool2cell (flag != REG_NOMATCH);
       else
@@ -1757,22 +1658,6 @@ nasl_exec (lex_ctxt * lexic, tree_cell * st)
 
 extern tree_cell *nasl_lint (lex_ctxt *, tree_cell *);
 
-/**
- * @brief This method is deprecated. Use @ref exec_nasl_script
- *        This function can be removed with the next major
- *        release after 2.0.
- *
- * @param script_infos The plugin as arglist. Has to be allocated, and at least
- *                     "preferences" sould be set.
- * @return Values < 0 
- */
-int
-execute_nasl_script (struct arglist *script_infos, const char *name,
-                     const char *cache_dir, int mode)
-{
-  return exec_nasl_script (script_infos, name, mode);
-}
-
 /**
  * @brief Execute a NASL script.
  *
@@ -1780,8 +1665,7 @@ execute_nasl_script (struct arglist *script_infos, const char *name,
  * bit #0 (1) is "description"
  * Bit #1 (2) is "parse only"
  *
- * @param script_infos The plugin as arglist. Has to be allocated, and at least
- *                     "preferences" sould be set.
+ * @param script_infos The plugin as arglist. Has to be allocated.
  * @param name         Filename.
  * @param mode         Bit field describing launch mode (description, parse
  *                     always signed).
@@ -1790,7 +1674,8 @@ execute_nasl_script (struct arglist *script_infos, const char *name,
  * error occurred.
  */
 int
-exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
+exec_nasl_script (struct arglist *script_infos, const char *name,
+                  const char *oid, int mode)
 {
   naslctxt ctx;
   nasl_func *pf;
@@ -1801,8 +1686,7 @@ exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
   gchar *newdir;
   char *old;
   tree_cell tc;
-  struct arglist *prefs = arg_get_value (script_infos, "preferences");
-  char *str;
+  const char *str;
   int to;
 
   srand48 (getpid () + getppid () + (long) time (NULL));
@@ -1813,13 +1697,11 @@ exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
   nasl_trace_fp = stderr;
 #endif
   if ((old = arg_get_value (script_infos, "script_name")) == NULL)
-    arg_add_value (script_infos, "script_name", ARG_STRING, strlen (name),
-                   estrdup (name));
+    arg_add_value (script_infos, "script_name", ARG_STRING, g_strdup (name));
   else
     {
-      efree (&old);
-      arg_set_value (script_infos, "script_name", strlen (name),
-                     estrdup (name));
+      g_free (old);
+      arg_set_value (script_infos, "script_name", g_strdup (name));
     }
 
   newdir = g_path_get_dirname (name);
@@ -1836,7 +1718,19 @@ exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
   if (mode & NASL_ALWAYS_SIGNED)
     ctx.always_authenticated = 1;
 
-  if (nasl_reload_or_parse (&ctx, name) < 0)
+  if (init_nasl_ctx (&ctx, name) == 0)
+    {
+      if (naslparse (&ctx))
+        {
+          log_legacy_write ("\n%s: Parse error at or near line %d\n",
+                            name, ctx.line_nb);
+          nasl_clean_ctx (&ctx);
+          g_chdir (old_dir);
+          g_free (old_dir);
+          return -1;
+        }
+    }
+  else
     {
       g_chdir (old_dir);
       g_free (old_dir);
@@ -1848,13 +1742,9 @@ exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
 #endif
   lexic = init_empty_lex_ctxt ();
   lexic->script_infos = script_infos;
+  lexic->oid = oid;
 
-  if (mode & NASL_ALWAYS_SIGNED)
-    lexic->authenticated = 1;
-  else
-    lexic->authenticated = ctx.authenticated;
-
-  str = arg_get_value (prefs, "checks_read_timeout");
+  str = prefs_get ("checks_read_timeout");
   if (str != NULL)
     to = atoi (str);
   else
@@ -1937,8 +1827,6 @@ exec_nasl_script (struct arglist *script_infos, const char *name, int mode)
       return -1;
     }
   g_free (old_dir);
-  if (mode & NASL_EXEC_DONT_CLEANUP)
-    return err;
 
   nasl_clean_ctx (&ctx);
   free_lex_ctxt (lexic);
