@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <glib.h>
@@ -21,9 +21,10 @@
 #include <ctype.h>              /* for isspace */
 #include <string.h>             /* for strlen */
 
+#include "../misc/prefs.h"      /* for prefs_get */
+
 #include "kb.h"                 /* for kb_item_get_str */
 #include "plugutils.h"          /* plug_get_host_fqdn */
-#include "system.h"             /* for emalloc */
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
@@ -37,16 +38,11 @@
 
 #include "nasl_http.h"
 
-#include "strutils.h"
 #include "www_funcs.h"          /* for build_encode_URL */
 
 /*-----------------[ http_* functions ]-------------------------------*/
 
 
-
-
-
-
 tree_cell *
 http_open_socket (lex_ctxt * lexic)
 {
@@ -75,7 +71,7 @@ _http_req (lex_ctxt * lexic, char *keyword)
   int cl;
   int al;
   char content_l_str[32];
-  struct kb_item **kb;
+  kb_t kb;
   int str_length = 0;
 
 
@@ -123,13 +119,13 @@ _http_req (lex_ctxt * lexic, char *keyword)
 
   if ((ver <= 0) || (ver == 11))
     {
-      char *hostname, *ua;
+      char *hostname, *ua, *hostheader;
 
-      hostname = (char *) plug_get_host_fqdn (script_infos);
+      hostname = plug_get_host_fqdn (script_infos);
       if (hostname == NULL)
         return NULL;
       ua = kb_item_get_str (kb, "http/user-agent");
-#define OPENVAS_USER_AGENT	"Mozilla/4.75 [en] (X11, U; OpenVAS)"
+#define OPENVAS_USER_AGENT	"Mozilla/5.0 [en] (X11, U; OpenVAS)"
       if (ua == NULL)
         ua = OPENVAS_USER_AGENT;
       else
@@ -140,19 +136,34 @@ _http_req (lex_ctxt * lexic, char *keyword)
             ua = OPENVAS_USER_AGENT;
         }
 
+      /* Servers should not have a problem with port 80 or 443 appended.
+       * RFC2616 allows to omit the port in which case the default port for
+       * that service is assumed.
+       * However, some servers like IIS/OWA wrongly respond with a "404"
+       * instead of a "200" in case the port is appended. Because of this,
+       * ports 80 and 443 are not appended.
+       */
+      if (port == 80 || port == 443)
+        hostheader = g_strdup (hostname);
+      else
+        hostheader = g_strdup_printf ("%s:%d", hostname, port);
+
       url = build_encode_URL (script_infos, keyword, NULL, item, "HTTP/1.1");
       str_length =
         strlen (url) + strlen (hostname) + al + cl + strlen (ua) + 1024;
-      str = emalloc (str_length);
+      str = g_malloc0 (str_length);
       /* NIDS evasion */
       g_snprintf (str, str_length, "%s\r\n\
 Connection: Close\r\n\
 Host: %s\r\n\
 Pragma: no-cache\r\n\
+Cache-Control: no-cache\r\n\
 User-Agent: %s\r\n\
 Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\r\n\
 Accept-Language: en\r\n\
-Accept-Charset: iso-8859-1,*,utf-8\r\n", url, hostname, ua);
+Accept-Charset: iso-8859-1,*,utf-8\r\n", url, hostheader, ua);
+      g_free (hostname);
+      g_free (hostheader);
     }
   else
     {
@@ -161,10 +172,10 @@ Accept-Charset: iso-8859-1,*,utf-8\r\n", url, hostname, ua);
         build_encode_URL (script_infos, keyword, NULL, item, "HTTP/1.0\r\n");
 
       str_length = strlen (url) + al + cl + 120;
-      str = emalloc (str_length);
+      str = g_malloc0 (str_length);
       g_strlcpy (str, url, str_length);
     }
-  efree (&url);
+  g_free (url);
 
   if (auth != NULL)
     {
@@ -249,16 +260,14 @@ http_put (lex_ctxt * lexic)
 tree_cell *
 cgibin (lex_ctxt * lexic)
 {
-  struct arglist *script_infos = lexic->script_infos;
-  struct arglist *prefs = arg_get_value (script_infos, "preferences");
-  char *path = prefs == NULL ? NULL : arg_get_value (prefs, "cgi_path");
+  const char *path = prefs_get ("cgi_path");
   tree_cell *retc;
 
   if (path == NULL)
     path = "/cgi-bin:/scripts";
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
-  retc->x.str_val = estrdup (path);
+  retc->x.str_val = g_strdup (path);
   retc->size = strlen (path);
 
   return retc;
