@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -21,9 +21,10 @@
                                and TH_FIN */
 
 #include <string.h>         /* for memset */
+#include <stdlib.h>         /* for getenv.  */
 
 #include "nvt_categories.h" /* for ACT_INIT */
-#include "plugutils.h"      /* for OPENVAS_ENCAPS_IP */
+#include "network.h"      /* for OPENVAS_ENCAPS_* */
 
 #include "nasl.h"
 #include "nasl_tree.h"
@@ -47,6 +48,11 @@
 #include "nasl_smb.h"
 #include "nasl_packet_forgery_v6.h"
 #include "nasl_builtin_plugins.h"
+#include "nasl_ssh.h"
+#include "nasl_snmp.h"
+#include "nasl_cert.h"
+#include "nasl_isotime.h"
+
 
 /* **************************************************************** */
 
@@ -67,13 +73,16 @@ static init_func libfuncs[] = {
   {"script_name", script_name, 1, {NULL}},
   {"script_version", script_version, 1, {NULL}},
   {"script_timeout", script_timeout, 1, {NULL}},
+
+  /* Can be removed once OpenVA-6 is retired and the command is
+     removed from any NVT */
   {"script_description", script_description, 999, {NULL}},
+
   {"script_copyright", script_copyright, 999, {NULL}},
   {"script_summary", script_summary, 999, {NULL}},
   {"script_category", script_category, 1, {NULL}},
   {"script_family", script_family, 999, {NULL}},
-  {"script_dependencie", script_dependencie, 999, {NULL}},
-  {"script_dependencies", script_dependencie, 999, {NULL}},
+  {"script_dependencies", script_dependencies, 999, {NULL}},
   {"script_require_keys", script_require_keys, 999, {NULL}},
   {"script_mandatory_keys", script_mandatory_keys, 999, {NULL}},
   {"script_require_ports", script_require_ports, 999, {NULL}},
@@ -95,23 +104,31 @@ static init_func libfuncs[] = {
   {"script_tag", script_tag, 0, {"name", "value", NULL}},
   {"get_preference", nasl_get_preference, 1, {NULL}},
   {"safe_checks", safe_checks, 0, {NULL}},
+  {"get_script_oid", get_script_oid, 0, {NULL}},
   {"replace_kb_item", replace_kb_item, 0, {"name", "value", NULL}},
   {"set_kb_item", set_kb_item, 0, {"name", "value", NULL}},
   {"get_kb_item", get_kb_item, 1, {NULL}},
   {"get_kb_fresh_item", get_kb_fresh_item, 1, {NULL}},
   {"get_kb_list", get_kb_list, 1, {NULL}},
-  {"security_warning", security_warning, 1,
+  {"security_message", security_message, 1,
    {"data", "port", "proto", "protocol", NULL}},
-  {"security_note", security_note, 1,
+
+  /* Once OpenVAS-5 is retired and the function calls are replaced
+     in all NASL scripts by security_message, these functions can removed
+     from the NASL API */
+  {"security_warning", security_message, 1,
    {"data", "port", "proto", "protocol", NULL}},
-  {"security_hole", security_hole, 1,
+  {"security_note", security_message, 1,
    {"data", "port", "proto", "protocol", NULL}},
+  {"security_hole", security_message, 1,
+   {"data", "port", "proto", "protocol", NULL}},
+
   {"log_message", log_message, 1, {"data", "port", "proto", "protocol", NULL}},
-  {"debug_message", debug_message, 1,
+  {"error_message", error_message, 1,
    {"data", "port", "proto", "protocol", NULL}},
 
   {"open_sock_tcp", nasl_open_sock_tcp, 1,
-   {"bufsz", "timeout", "transport", NULL}},
+   {"bufsz", "priority", "timeout", "transport", NULL}},
   {"open_sock_udp", nasl_open_sock_udp, 1, {NULL}},
   {"open_priv_sock_tcp", nasl_open_priv_sock_tcp, 0,
    {"dport", "sport", "timeout", NULL}},
@@ -121,10 +138,21 @@ static init_func libfuncs[] = {
   {"recv", nasl_recv, 0, {"length", "min", "socket", "timeout", NULL}},
   {"recv_line", nasl_recv_line, 0, {"length", "socket", "timeout", NULL}},
   {"send", nasl_send, 0, {"data", "length", "option", "socket", NULL}},
+  {"socket_negotiate_ssl", nasl_socket_negotiate_ssl, 0,
+   {"socket", "transport", NULL}},
+  {"socket_get_cert", nasl_socket_get_cert, 0, {"socket", NULL}},
+  {"socket_get_ssl_version", nasl_socket_get_ssl_version, 0, {"socket", NULL}},
+  {"socket_get_ssl_ciphersuite", nasl_socket_get_ssl_ciphersuite, 0,
+   {"socket", NULL}},
+  {"socket_get_ssl_session_id", nasl_socket_get_ssl_session_id, 0,
+   {"socket", NULL}},
+  {"socket_get_ssl_compression", nasl_socket_get_ssl_compression, 0,
+   {"socket", NULL}},
   {"close", nasl_close_socket, 1, {NULL}},
   {"join_multicast_group", nasl_join_multicast_group, 1, {NULL}},
   {"leave_multicast_group", nasl_leave_multicast_group, 1, {NULL}},
   {"get_source_port", nasl_get_source_port, 1, {NULL}}, /* DOC! */
+  {"get_sock_info", nasl_get_sock_info, 2, {"asstring", NULL}},
 
   {"cgibin", cgibin, 0, {NULL}},
   {"http_open_socket", http_open_socket, 1, {NULL}},
@@ -149,7 +177,7 @@ static init_func libfuncs[] = {
   {"scanner_get_port", nasl_scanner_get_port, 1, {NULL}},
   {"islocalhost", nasl_islocalhost, 0, {NULL}},
   {"islocalnet", nasl_islocalnet, 0, {NULL}},
-  {"get_port_transport", get_port_transport, 1, {NULL}},
+  {"get_port_transport", get_port_transport, 1, {"asstring", NULL}},
   {"this_host", nasl_this_host, 0, {NULL}},
   {"this_host_name", nasl_this_host_name, 0, {NULL}},
 
@@ -357,6 +385,39 @@ static init_func libfuncs[] = {
   {"rsa_sign", nasl_rsa_sign, 0, {"d", "data", "e", "n", NULL}},
   {"dsa_do_sign", nasl_dsa_do_sign, 0,
    {"data", "g", "p", "priv", "pub", "q", NULL}},
+  {"gunzip", nasl_gunzip, 0, {"data", "len", NULL}},
+  {"gzip", nasl_gzip, 0, {"data", "len", NULL}},
+
+#ifdef HAVE_NETSNMP
+  {"snmpv1_get", nasl_snmpv1_get, 0, {"community", "oid", "port", "protocol",
+                                      NULL }},
+  {"snmpv3_get", nasl_snmpv3_get, 0, {"authpass", "authproto", "oid", "port",
+                                      "privpass", "privproto", "username",
+                                      NULL }},
+#endif /* HAVE_NETSNMP */
+
+  {"ssh_connect", nasl_ssh_connect, 0, {"port", "socket", NULL }},
+  {"ssh_disconnect", nasl_ssh_disconnect, 1, {NULL }},
+  {"ssh_session_id_from_sock", nasl_ssh_session_id_from_sock, 1, {NULL }},
+  {"ssh_get_sock", nasl_ssh_get_sock, 1, {NULL }},
+  {"ssh_set_login", nasl_ssh_set_login, 1, {"login", NULL }},
+  {"ssh_userauth", nasl_ssh_userauth, 1, {"login", "password", NULL }},
+  {"ssh_request_exec", nasl_ssh_request_exec,
+   1, {"cmd", "stderr", "stdout",NULL }},
+  {"ssh_shell_open", nasl_ssh_shell_open, 1, { NULL }},
+  {"ssh_shell_read", nasl_ssh_shell_read, 1, { NULL }},
+  {"ssh_shell_write", nasl_ssh_shell_write, 1, { "cmd" }},
+  {"ssh_get_issue_banner", nasl_ssh_get_issue_banner, 1, {NULL }},
+#if LIBSSH_VERSION_INT >= SSH_VERSION_INT (0, 6, 0)
+  {"ssh_get_server_banner", nasl_ssh_get_server_banner, 1, {NULL }},
+#endif
+  {"ssh_get_auth_methods", nasl_ssh_get_auth_methods, 1, {NULL }},
+
+#ifdef HAVE_LIBKSBA
+  {"cert_open", nasl_cert_open, 1, {"errorvar", NULL }},
+  {"cert_close", nasl_cert_close, 1, {NULL }},
+  {"cert_query", nasl_cert_query, 2, {"idx", NULL }},
+#endif /*HAVE_LIBKSBA*/
 
   {"pread", nasl_pread, 0, {"argv", "cd", "cmd", "nice", NULL}},
   {"find_in_path", nasl_find_in_path, 1, {NULL}},
@@ -374,12 +435,6 @@ static init_func libfuncs[] = {
   {"file_write", nasl_file_write, 0, {"data", "fp", NULL}},
   {"file_seek", nasl_file_seek, 0, {"fp", "offset", NULL}},
 
-  {"shared_socket_register", nasl_shared_socket_register, 0,
-   {"name", "socket", NULL}},
-  {"shared_socket_acquire", nasl_shared_socket_acquire, 1, {NULL}},
-  {"shared_socket_release", nasl_shared_socket_release, 1, {NULL}},
-  {"shared_socket_destroy", nasl_shared_socket_destroy, 1, {NULL}},
-
   {"wmi_versioninfo", nasl_wmi_versioninfo, 0, {NULL}},
   {"wmi_connect", nasl_wmi_connect, 4,
    {"ns", "password", "username", NULL}},
@@ -406,6 +461,18 @@ static init_func libfuncs[] = {
    {"hive", "key", "val_name", "wmi_handle", NULL}},
   {"wmi_reg_get_qword_val", nasl_wmi_reg_get_qword_val, 0,
    {"hive", "key", "val_name", "wmi_handle", NULL}},
+  {"wmi_reg_set_dword_val", nasl_wmi_reg_set_dword_val, 0,
+   {"hive", "key", "val", "val_name", "wmi_handle"}},
+  {"wmi_reg_set_qword_val", nasl_wmi_reg_set_qword_val, 0,
+   {"hive", "key", "val", "val_name", "wmi_handle"}},
+  {"wmi_reg_set_ex_string_val", nasl_wmi_reg_set_ex_string_val, 0,
+   {"hive", "key", "val", "val_name", "wmi_handle"}},
+  {"wmi_reg_set_string_val", nasl_wmi_reg_set_string_val, 0,
+   {"hive", "key", "val", "val_name", "wmi_handle"}},
+  {"wmi_reg_create_key", nasl_wmi_reg_create_key, 0,
+   {"hive", "key", "wmi_handle"}},
+  {"wmi_reg_delete_key", nasl_wmi_reg_delete_key, 0,
+   {"hive", "key", "wmi_handle"}},
 
   {"smb_versioninfo", nasl_smb_versioninfo, 0, {NULL}},
   {"smb_connect", nasl_smb_connect, 0,
@@ -418,6 +485,8 @@ static init_func libfuncs[] = {
    {"filename", "smb_handle", NULL}},
   {"smb_file_trustee_rights", nasl_smb_file_trustee_rights, 0,
    {"filename", "smb_handle", NULL}},
+  {"win_cmd_exec", nasl_win_cmd_exec, 0,
+   {"cmd", "password", "username"}},
 
   {"scan_phase", scan_phase, 0, {NULL}},
   {"network_targets", network_targets, 0, {NULL}},
@@ -425,6 +494,13 @@ static init_func libfuncs[] = {
   {"plugin_run_find_service", plugin_run_find_service, 0, {NULL}},
   {"plugin_run_openvas_tcp_scanner", plugin_run_openvas_tcp_scanner, 0, {NULL}},
   {"plugin_run_synscan", plugin_run_synscan, 0, {NULL}},
+  {"plugin_run_nmap", plugin_run_nmap, 0, {NULL}},
+
+  {"isotime_now",      nasl_isotime_now, 0, {NULL}},
+  {"isotime_is_valid", nasl_isotime_is_valid, 1, {NULL}},
+  {"isotime_scan",     nasl_isotime_scan, 1, {NULL}},
+  {"isotime_print",    nasl_isotime_print, 1, {NULL}},
+  {"isotime_add",      nasl_isotime_add, 1, {"days", "seconds", "years", NULL}},
 
   {NULL, NULL, 0, {NULL}}
 };
@@ -465,6 +541,8 @@ static struct
   {
   "IPPROTO_IGMP", IPPROTO_IGMP},
   {
+  "ENCAPS_AUTO", OPENVAS_ENCAPS_AUTO},
+  {
   "ENCAPS_IP", OPENVAS_ENCAPS_IP},
   {
   "ENCAPS_SSLv23", OPENVAS_ENCAPS_SSLv23},
@@ -475,9 +553,13 @@ static struct
   {
   "ENCAPS_TLSv1", OPENVAS_ENCAPS_TLSv1},
   {
-  "NASL_LEVEL", NASL_LEVEL},    /* deprecated: can be removed once openvas-plugins < 1.0.6 is deprecated */
+  "ENCAPS_TLSv11", OPENVAS_ENCAPS_TLSv11},
   {
-  "OPENVAS_NASL_LEVEL", OPENVAS_NASL_LEVEL},
+  "ENCAPS_TLSv12", OPENVAS_ENCAPS_TLSv12},
+  {
+  "ENCAPS_TLScustom", OPENVAS_ENCAPS_TLScustom},
+  {
+  "ENCAPS_MAX", OPENVAS_ENCAPS_MAX},
   {
   "TH_FIN", TH_FIN},
   {
@@ -533,6 +615,13 @@ static struct
   {
   "EUNKNOWN", NASL_ERR_EUNKNOWN},
   {
+  /* Since OpenVAS-8, libssh is mandatory. To maintain compatibility of
+     the NVT feed with older versions, this variable needs to be set.
+     Once OpenVAS-7 is retired, this setting of the variable can be removed
+     and also any occurances in the NVTs, which should lead to some
+     significant NASL code removals. */
+  "_HAVE_LIBSSH", 1},
+  {
 NULL, 0},};
 
 /* See also in exec.c:
@@ -550,7 +639,6 @@ init_nasl_library (lex_ctxt * lexic)
 {
   int i, j, c;
   nasl_func *pf;
-  named_nasl_var *v;
   tree_cell tc;
   const char **p, *q;
 
@@ -559,7 +647,7 @@ init_nasl_library (lex_ctxt * lexic)
     {
       if ((pf = insert_nasl_func (lexic, libfuncs[i].name, NULL)) == NULL)
         {
-          nasl_perror (lexic, "init_nasl2_library: could not define fct '%s'\n",
+          nasl_perror (lexic, "init_nasl_library: could not define fct '%s'\n",
                        libfuncs[i].name);
           continue;
         }
@@ -571,7 +659,7 @@ init_nasl_library (lex_ctxt * lexic)
         {
           if (q != NULL && strcmp (q, *p) > 0)
             nasl_perror (lexic,
-                         "init_nasl2_library: unsorted args for function %s: %s > %s\n",
+                         "init_nasl_library: unsorted args for function %s: %s > %s\n",
                          libfuncs[i].name, q, (*p));
           q = (*p);
           p++;
@@ -587,9 +675,9 @@ init_nasl_library (lex_ctxt * lexic)
   for (i = 0; i < sizeof (libivars) / sizeof (libivars[0]) - 1; i++)
     {
       tc.x.i_val = libivars[i].val;
-      if ((v = add_named_var_to_ctxt (lexic, libivars[i].name, &tc)) == NULL)
+      if (add_named_var_to_ctxt (lexic, libivars[i].name, &tc) == NULL)
         {
-          nasl_perror (lexic, "init_nasl2_library: could not define var '%s'\n",
+          nasl_perror (lexic, "init_nasl_library: could not define var '%s'\n",
                        libivars[i].name);
           continue;
         }
@@ -602,9 +690,9 @@ init_nasl_library (lex_ctxt * lexic)
     {
       tc.x.str_val = (char *) libsvars[i].val;
       tc.size = strlen (libsvars[i].val);
-      if ((v = add_named_var_to_ctxt (lexic, libsvars[i].name, &tc)) == NULL)
+      if (add_named_var_to_ctxt (lexic, libsvars[i].name, &tc) == NULL)
         {
-          nasl_perror (lexic, "init_nasl2_library: could not define var '%s'\n",
+          nasl_perror (lexic, "init_nasl_library: could not define var '%s'\n",
                        libsvars[i].name);
           continue;
         }
@@ -613,7 +701,7 @@ init_nasl_library (lex_ctxt * lexic)
 
   // Add the "NULL" variable
   if (add_named_var_to_ctxt (lexic, "NULL", NULL) == NULL)
-    nasl_perror (lexic, "init_nasl2_library: could not define var 'NULL'\n");
+    nasl_perror (lexic, "init_nasl_library: could not define var 'NULL'\n");
 
   return c;
 }
