@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
  /*
@@ -32,7 +32,7 @@
 #include "kb.h"                 /* for KB_TYPE_STR */
 #include "network.h"            /* read_stream_connection_min */
 #include "plugutils.h"          /* plug_get_host_open_port */
-#include "system.h"             /* for emalloc */
+#include "openvas_compress.h"
 
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
@@ -41,7 +41,6 @@
 #include "nasl_lex_ctxt.h"
 #include "exec.h"
 
-#include "strutils.h"
 #include "nasl_packet_forgery.h"
 #include "nasl_debug.h"
 #include "nasl_misc_funcs.h"
@@ -49,6 +48,8 @@
 
 #define uint32 unsigned int
 
+#define NASL_EXIT_DEPRECATED 66
+#define NASL_EXIT_NOTVULN    99
 
 /*---------------------------------------------------------------------*/
 tree_cell *
@@ -196,7 +197,7 @@ nasl_telnet_init (lex_ctxt * lexic)
     n += n2;
   retc = alloc_typed_cell (CONST_DATA);
   retc->size = n;
-  retc->x.str_val = nasl_strndup ((char *) buffer, n);
+  retc->x.str_val = g_memdup (buffer, n + 1);
 #undef iac
 #undef data
 #undef option
@@ -222,11 +223,11 @@ nasl_start_denial (lex_ctxt * lexic)
       if (soc >= 0)
         {
           if (arg_get_value (script_infos, "denial_port") != 0)
-            arg_set_value (script_infos, "denial_port", sizeof (gpointer),
+            arg_set_value (script_infos, "denial_port",
                            GSIZE_TO_POINTER (port));
           else
             arg_add_value (script_infos, "denial_port", ARG_INT,
-                           sizeof (gpointer), GSIZE_TO_POINTER (port));
+                           GSIZE_TO_POINTER (port));
 
           close_stream_connection (soc);
 
@@ -239,10 +240,10 @@ nasl_start_denial (lex_ctxt * lexic)
     alive = p->x.i_val;
 
   if (arg_get_value (script_infos, "tcp_ping_result") != 0)
-    arg_set_value (script_infos, "tcp_ping_result", sizeof (gpointer),
+    arg_set_value (script_infos, "tcp_ping_result",
                    GSIZE_TO_POINTER (alive));
   else
-    arg_add_value (script_infos, "tcp_ping_result", ARG_INT, sizeof (gpointer),
+    arg_add_value (script_infos, "tcp_ping_result", ARG_INT,
                    GSIZE_TO_POINTER (alive));
 
   deref_cell (p);
@@ -253,22 +254,20 @@ nasl_start_denial (lex_ctxt * lexic)
 tree_cell *
 nasl_end_denial (lex_ctxt * lexic)
 {
-  int port =
-    GPOINTER_TO_SIZE (arg_get_value (lexic->script_infos, "denial_port"));
+  int port = arg_get_value_int (lexic->script_infos, "denial_port");
   int soc;
   int to = lexic->recv_timeout;
   struct arglist *script_infos = lexic->script_infos;
   tree_cell *retc = NULL;
 
-  /* 
+  /*
    * We must wait the time the DoS does its effect
    */
   sleep (10);
 
   if (!port)
     {
-      int ping =
-        GPOINTER_TO_SIZE (arg_get_value (script_infos, "tcp_ping_result"));
+      int ping = arg_get_value_int (script_infos, "tcp_ping_result");
 
       if (ping)
         return nasl_tcp_ping (lexic);
@@ -315,15 +314,33 @@ nasl_dump_ctxt (lex_ctxt * lexic)
 
 
 
+static void
+simple_register_host_detail(lex_ctxt *lexic, char *name, char *value)
+{
+  char detail[128];
+  const char *oid = lexic->oid;
 
+  plug_set_key (lexic->script_infos, "HostDetails", ARG_STRING, name);
+  plug_set_key (lexic->script_infos, "HostDetails/NVT", ARG_STRING, (void *)oid);
+
+  g_snprintf (detail, sizeof (detail), "HostDetails/NVT/%s/%s", oid, name);
+  plug_set_key (lexic->script_infos, detail, ARG_STRING, value);
+}
 
 tree_cell *
 nasl_do_exit (lex_ctxt * lexic)
 {
-  int x = get_int_var_by_num (lexic, 0, 0);
+  int retcode = get_int_var_by_num (lexic, 0, 0);
   tree_cell *retc = alloc_tree_cell (0, NULL);
+
   retc->type = CONST_INT;
-  retc->x.i_val = x;
+  retc->x.i_val = retcode;
+
+  if (retcode == NASL_EXIT_NOTVULN)
+    simple_register_host_detail(lexic, "EXIT_CODE", "EXIT_NOTVULN");
+
+//  if (retcode == NASL_EXIT_DEPRECATED)
+    // This return code is reserved for future handling.
 
   while (lexic != NULL)
     {
@@ -368,7 +385,7 @@ nasl_make_list (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   for (i = vi = 0; (v = nasl_get_var_by_num (&lexic->ctx_vars, vi, 0)) != NULL;
        vi++)
@@ -421,7 +438,7 @@ nasl_make_list (lex_ctxt * lexic)
 
 /*
  * This function takes any _even_ number of arguments and makes
- * an array from them. In each pair, the 1st argument is the index, the 
+ * an array from them. In each pair, the 1st argument is the index, the
  * 2nd the value.
  * Illegal types are droped with a warning
  */
@@ -430,16 +447,16 @@ tree_cell *
 nasl_make_array (lex_ctxt * lexic)
 {
   tree_cell *retc = NULL;
-  int i, vi;
+  int vi;
   anon_nasl_var *v, *v2;
   nasl_array *a;
 
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
-  i = vi = 0;
+  vi = 0;
   while ((v = nasl_get_var_by_num (&lexic->ctx_vars, vi++, 0)) != NULL)
     {
       v2 = nasl_get_var_by_num (&lexic->ctx_vars, vi++, 0);
@@ -488,7 +505,7 @@ nasl_keys (lex_ctxt * lexic)
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = DYN_ARRAY;
-  retc->x.ref_val = a2 = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a2 = g_malloc0 (sizeof (nasl_array));
 
   bzero (&myvar, sizeof (myvar));
 
@@ -583,7 +600,7 @@ nasl_typeof (lex_ctxt * lexic)
         break;
       }
   retc->size = strlen (s);
-  retc->x.str_val = emalloc (retc->size);
+  retc->x.str_val = g_malloc0 (retc->size);
   strcpy (retc->x.str_val, s);  /* Flawfinder: ignore */
   return retc;
 }
@@ -634,7 +651,7 @@ nasl_func_named_args (lex_ctxt * lexic)
     }
 
   retc = alloc_typed_cell (DYN_ARRAY);
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
 
   memset (&v, 0, sizeof (v));
   v.var_type = VAR2_STRING;
@@ -792,7 +809,7 @@ nasl_gettimeofday (lex_ctxt * lexic)
   sprintf (str, "%u.%06u", (unsigned int) t.tv_sec, (unsigned int) t.tv_usec);
   retc = alloc_typed_cell (CONST_DATA);
   retc->size = strlen (str);
-  retc->x.str_val = emalloc (retc->size);
+  retc->x.str_val = g_malloc0 (retc->size);
   strcpy (retc->x.str_val, str);        /* Flawfinder: ignore */
   return retc;
 }
@@ -826,7 +843,7 @@ nasl_localtime (lex_ctxt * lexic)
     }
 
   retc = alloc_typed_cell (DYN_ARRAY);
-  retc->x.ref_val = a = emalloc (sizeof (nasl_array));
+  retc->x.ref_val = a = g_malloc0 (sizeof (nasl_array));
   memset (&v, 0, sizeof (v));
   v.var_type = VAR2_INT;
 
@@ -891,15 +908,10 @@ tree_cell *
 nasl_open_sock_kdc (lex_ctxt * lexic)
 {
   tree_cell *retc;
-  int timeout = 30;
-  int port = 88;
-  char *hostname = NULL;        /* Domain name for windows */
-  int tcp = 0;
-  int ret;
+  int ret, type;
+  int timeout = 30, port = 88, tcp = 0;
+  char *hostname = NULL, *port_str, *tcp_str;   /* Domain name for windows */
   struct arglist *script_infos;
-  int type;
-
-
 
   script_infos = lexic->script_infos;
 
@@ -907,14 +919,16 @@ nasl_open_sock_kdc (lex_ctxt * lexic)
   if (!hostname || type != KB_TYPE_STR)
     return NULL;
 
-  port =
-    GPOINTER_TO_SIZE (plug_get_key (script_infos, "Secret/kdc_port", &type));
+  port_str = plug_get_key (script_infos, "Secret/kdc_port", &type);
+  port = GPOINTER_TO_SIZE (port_str);
+  g_free (port_str);
   if (port <= 0 || type != KB_TYPE_INT)
     return NULL;
 
 
-  tcp =
-    GPOINTER_TO_SIZE (plug_get_key (script_infos, "Secret/kdc_use_tcp", &type));
+  tcp_str = plug_get_key (script_infos, "Secret/kdc_use_tcp", &type);
+  tcp = GPOINTER_TO_SIZE (tcp_str);
+  g_free (tcp_str);
   if (tcp < 0 || type != KB_TYPE_INT)
     tcp = 0;
 
@@ -922,6 +936,7 @@ nasl_open_sock_kdc (lex_ctxt * lexic)
     ret = open_sock_opt_hn (hostname, port, SOCK_DGRAM, IPPROTO_UDP, timeout);
   else
     ret = open_sock_opt_hn (hostname, port, SOCK_STREAM, IPPROTO_TCP, timeout);
+  g_free (hostname);
 
   if (ret < 0)
     return NULL;
@@ -931,6 +946,59 @@ nasl_open_sock_kdc (lex_ctxt * lexic)
   return retc;
 }
 
+tree_cell *
+nasl_gunzip (lex_ctxt * lexic)
+{
+  tree_cell *retc;
+  void *data, *uncompressed;
+  unsigned long datalen, uncomplen;
+
+  data = get_str_local_var_by_name (lexic, "data");
+  if (data == NULL)
+    return NULL;
+  datalen = get_var_size_by_name (lexic, "data");
+  if (datalen <= 0)
+    return NULL;
+
+  uncompressed = openvas_uncompress (data, datalen, &uncomplen);
+  if (uncompressed == NULL)
+    return NULL;
+
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->size = uncomplen;
+  retc->x.str_val = uncompressed;
+
+  return retc;
+}
+
+tree_cell *
+nasl_gzip (lex_ctxt * lexic)
+{
+  tree_cell *retc;
+  void *data, *compressed;
+  unsigned long datalen, complen;
+
+  data = get_str_local_var_by_name (lexic, "data");
+  if (data == NULL)
+    return NULL;
+  datalen = get_var_size_by_name (lexic, "data");
+  if (datalen <= 0)
+    return NULL;
+
+  compressed = openvas_compress (data, datalen, &complen);
+  if (compressed == NULL)
+    return NULL;
+
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->size = complen;
+  retc->x.str_val = compressed;
+
+  return retc;
+}
+
+
 tree_cell *
 nasl_dec2str (lex_ctxt * lexic)
 {
@@ -941,7 +1009,7 @@ nasl_dec2str (lex_ctxt * lexic)
       nasl_perror (lexic, "Syntax : dec2str(num:<n>)\n");
       return NULL;
     }
-  char *ret = emalloc (sizeof (num));
+  char *ret = g_malloc0 (sizeof (num));
   SIVAL (ret, 0, num);
   tree_cell *retc;
   retc = alloc_tree_cell (0, NULL);
