@@ -1,6 +1,14 @@
-/* Nessus Attack Scripting Language 
+/* openvas-libraries/nasl/nasl_socket.c
+ * $Id$
+ * Description: NASL socket API
  *
+ * Authors:
+ * Unknown
+ * Werner Koch <wk@gnupg.org>
+ *
+ * Copyright:
  * Copyright (C) 2002 - 2004 Tenable Network Security
+ * Copyright (C) 2012 Greenbone Networks GmbH
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -13,19 +21,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
-
-
-/** @file
-  * This file contains all the functions related to the handling of the sockets
-  * within a NASL script - namely, this is the implementation
-  * of open_(priv_)?sock_(udp|tcp)(), send(), recv(), recv_line() and
-  * close().
-  */
-
+/**
+ * @file nasl_socket.c
+ *
+ * @brief The NASL socket API.
+ *
+ * This file contains all the functions related to the handling of the
+ * sockets within a NASL script - for example the implementation of
+ * the NASL built-ins open_sock_tcp, send, recv, recv_line, and close.
+ */
 
 
 /*--------------------------------------------------------------------------*/
@@ -36,9 +44,12 @@
 #include <string.h>             /* for bzero */
 #include <unistd.h>             /* for close */
 
-#include "network.h"            /* for set_socket_source_addr */
+#include <gnutls/gnutls.h>
+
+#include "network.h"
+#include "openvas_networking.h" /* for openvas_source_set_socket */
 #include "plugutils.h"          /* for plug_get_host_ip */
-#include "system.h"             /* for efree */
+#include "../misc/openvas_logging.h"
 
 #include "nasl.h"
 
@@ -49,8 +60,6 @@
 #include "nasl_lex_ctxt.h"
 #include "exec.h"
 
-#include "strutils.h"
-
 #include "nasl_packet_forgery.h"
 #include "nasl_debug.h"
 
@@ -99,7 +108,7 @@ block_socket (int soc)
  * fails. The point is to take care of packets lost en route.
  *
  * To do this, we store a copy of the data sent by a given socket
- * each time send() is called, and we re-send() it each time 
+ * each time send() is called, and we re-send() it each time
  * recv() is called and fails
  *
  */
@@ -123,7 +132,7 @@ add_udp_data (struct arglist *script_infos, int soc, char *data, int len)
   if (udp_data == NULL)
     {
       udp_data = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);
-      arg_add_value (script_infos, "udp_data", ARG_PTR, -1, udp_data);
+      arg_add_value (script_infos, "udp_data", ARG_PTR, udp_data);
     }
 
   g_hash_table_replace (udp_data, (gpointer)key, (gpointer)data_record);
@@ -150,7 +159,8 @@ rm_udp_data (struct arglist *script_infos, int soc)
 {
   GHashTable * udp_data = arg_get_value (script_infos, "udp_data");
 
-  g_hash_table_remove (udp_data, (gconstpointer)&soc);
+  if (udp_data)
+    g_hash_table_remove (udp_data, (gconstpointer)&soc);
 }
 
 
@@ -225,7 +235,7 @@ tryagain:
   if (current_sport < 128 && sport < 0)
     return NULL;
   e =
-    set_socket_source_addr (sock, sport > 0 ? sport : current_sport--, family);
+    openvas_source_set_socket (sock, sport > 0 ? sport : current_sport--, family);
 
   /*
    * bind() failed - try again on a lower port
@@ -303,8 +313,8 @@ tryagain:
 
   if (getsockopt (sock, SOL_SOCKET, SO_ERROR, &opt, &opt_sz) < 0)
     {
-      fprintf (stderr, "[%d] open_priv_sock()->getsockopt() failed : %s\n",
-               getpid (), strerror (errno));
+      log_legacy_write ("[%d] open_priv_sock()->getsockopt() failed : %s\n",
+                        getpid (), strerror (errno));
       close (sock);
       return NULL;
     }
@@ -329,7 +339,7 @@ tryagain:
     }
 
   if (proto == IPPROTO_TCP)
-    sock = openvas_register_connection (sock, NULL, NULL);
+    sock = openvas_register_connection (sock, NULL, NULL, OPENVAS_ENCAPS_IP);
 
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_INT;
@@ -358,7 +368,9 @@ nasl_open_sock_tcp_bufsz (lex_ctxt * lexic, int bufsz)
 {
   int soc = -1;
   struct arglist *script_infos = lexic->script_infos;
-  int to, port, transport = -1;
+  int to, port;
+  int transport = -1;
+  const char *priority;
   tree_cell *retc;
 
   to = get_int_local_var_by_name (lexic, "timeout", lexic->recv_timeout * 2);
@@ -366,6 +378,20 @@ nasl_open_sock_tcp_bufsz (lex_ctxt * lexic, int bufsz)
     to = 10;
 
   transport = get_int_local_var_by_name (lexic, "transport", -1);
+
+  if (transport == OPENVAS_ENCAPS_TLScustom)
+    {
+      int type;
+      priority = get_str_local_var_by_name (lexic, "priority");
+      if (!priority)
+        priority = NULL;
+      type = get_local_var_type_by_name (lexic, "priority");
+      if (type != VAR2_STRING && type != VAR2_DATA)
+        priority = NULL;
+    }
+  else
+    priority = NULL;
+
   if (bufsz < 0)
     bufsz = get_int_local_var_by_name (lexic, "bufsz", 0);
 
@@ -373,10 +399,16 @@ nasl_open_sock_tcp_bufsz (lex_ctxt * lexic, int bufsz)
   if (port < 0)
     return NULL;
 
+  /* If "transport" has not been given, use auto detection if enabled
+     in the KB. if "transport" has been given with a value of 0 force
+     autodetection reagardless of what the KB tells.  */
   if (transport < 0)
-    soc = open_stream_auto_encaps (script_infos, port, to);
+    soc = open_stream_auto_encaps_ext (script_infos, port, to, 0);
+  else if (transport == 0)
+    soc = open_stream_auto_encaps_ext (script_infos, port, to, 1);
   else
-    soc = open_stream_connection (script_infos, port, transport, to);
+    soc = open_stream_connection_ext (script_infos, port, transport, to,
+                                      priority);
   if (bufsz > 0 && soc >= 0)
     {
       if (stream_set_buffer (soc, bufsz) < 0)
@@ -390,16 +422,57 @@ nasl_open_sock_tcp_bufsz (lex_ctxt * lexic, int bufsz)
   return retc;
 }
 
+/**
+ * @brief Open a TCP socket to the target host
+ * @naslfn{open_sock_tcp}
+ *
+ * This function is used to create a TCP connection to the target
+ * host.  It requires the port number as its argument and has various
+ * optional named arguments to control encapsulation, timeout and
+ * buffering.
+ *
+ * @nasluparam
+ *
+ * - A non-negative integer with the TCP port number.
+ *
+ * @naslnparam
+ *
+ * - @a bufsz An integer with the the size buffer size.  Note that by
+ *    default, no buffering is used.
+ *
+ * - @a timeout An integer with the timeout value in seconds.  The
+ *    default timeout is controlled by a global value.
+ *
+ * - @a transport One of the ENCAPS_* constants to force a specific
+ *    encapsulation mode or force trying of all modes (ENCAPS_AUTO).
+ *    This is for example useful to select a specific TLS or SSL
+ *    version or use specific TLS connection setup priorities.  See
+ *    \ref get_port_transport for a description of the ENCAPS
+ *    constants.
+ *
+ * - @a priority A string value with priorities for an TLS
+ *    encapsulation.  For the syntax of the priority string see the
+ *    GNUTLS manual.  This argument is only used in @a
+ *    ENCAPS_TLScustom encapsulation.
+ *
+ * @naslret A positive integer as a NASL socket, 0 on connection error or
+ * NULL on other errors.
+ *
+ * @param[in] lexic  Lexical context of the NASL interpreter.
+ *
+ * @return A tree cell.
+ */
 tree_cell *
 nasl_open_sock_tcp (lex_ctxt * lexic)
 {
   return nasl_open_sock_tcp_bufsz (lexic, -1);
 }
 
+
 /*
  * Opening a UDP socket is a little more tricky, since
  * UDP works in a way which is different from TCP...
- * 
+ *
  * Our goal is to hide this difference for the end-user
  */
 tree_cell *
@@ -428,7 +501,7 @@ nasl_open_sock_udp (lex_ctxt * lexic)
       soca.sin_family = AF_INET;
 
       soc = socket (AF_INET, SOCK_DGRAM, 0);
-      set_socket_source_addr (soc, 0, AF_INET);
+      openvas_source_set_socket (soc, 0, AF_INET);
       connect (soc, (struct sockaddr *) &soca, sizeof (soca));
     }
   else
@@ -439,7 +512,7 @@ nasl_open_sock_udp (lex_ctxt * lexic)
       soca6.sin6_family = AF_INET6;
 
       soc = socket (AF_INET6, SOCK_DGRAM, 0);
-      set_socket_source_addr (soc, 0, AF_INET6);
+      openvas_source_set_socket (soc, 0, AF_INET6);
       connect (soc, (struct sockaddr *) &soca6, sizeof (soca6));
     }
 
@@ -450,6 +523,141 @@ nasl_open_sock_udp (lex_ctxt * lexic)
   return retc;
 }
 
+tree_cell *
+nasl_socket_negotiate_ssl (lex_ctxt * lexic)
+{
+  int soc, transport, ret;
+  tree_cell *retc;
+
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  transport = get_int_local_var_by_name (lexic, "transport",
+                                         OPENVAS_ENCAPS_TLScustom);
+  if (soc < 0)
+    {
+      nasl_perror (lexic, "socket_ssl_negotiate: Erroneous socket value %d\n",
+                   soc);
+      return NULL;
+    }
+  if (transport == -1)
+    transport = OPENVAS_ENCAPS_TLScustom;
+  else if (!IS_ENCAPS_SSL (transport))
+    {
+      nasl_perror (lexic, "socket_ssl_negotiate: Erroneous transport value %d\n",
+                   transport);
+      return NULL;
+    }
+  ret = socket_negotiate_ssl (soc, transport, lexic->script_infos);
+  if (ret < 0)
+    return NULL;
+
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_INT;
+  retc->x.i_val = ret;
+  return retc;
+}
+
+tree_cell *
+nasl_socket_get_cert (lex_ctxt * lexic)
+{
+  int soc, cert_len = 0;
+  tree_cell *retc;
+  void *cert;
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  if (soc < 0)
+    {
+      nasl_perror (lexic, "socket_get_cert: Erroneous socket value %d\n",
+                   soc);
+      return NULL;
+    }
+  socket_get_cert (soc, &cert, &cert_len);
+  if (cert_len <= 0)
+    return NULL;
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->x.str_val = cert;
+  retc->size = cert_len;
+  return retc;
+}
+
+tree_cell *
+nasl_socket_get_ssl_session_id (lex_ctxt * lexic)
+{
+  int soc;
+  size_t sid_len = 0;
+  tree_cell *retc;
+  void *sid;
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  if (soc < 0)
+    {
+      nasl_perror (lexic, "socket_get_cert: Erroneous socket value %d\n",
+                   soc);
+      return NULL;
+    }
+  socket_get_ssl_session_id (soc, &sid, &sid_len);
+  if (sid == NULL || sid_len == 0)
+    return NULL;
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_DATA;
+  retc->x.str_val = sid;
+  retc->size = sid_len;
+  return retc;
+}
+
+tree_cell *
+nasl_socket_get_ssl_compression (lex_ctxt * lexic)
+{
+  int soc;
+  tree_cell *retc;
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  if (soc < 0)
+    {
+      nasl_perror (lexic, "socket_get_cert: Erroneous socket value %d\n",
+                   soc);
+      return NULL;
+    }
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_INT;
+  retc->x.i_val = socket_get_ssl_compression (soc);
+  return retc;
+}
+
+tree_cell *
+nasl_socket_get_ssl_version (lex_ctxt * lexic)
+{
+  int soc;
+  int version;
+  tree_cell *retc;
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  version = socket_get_ssl_version (soc);
+  if (version < 0)
+    return NULL;
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_INT;
+  retc->x.i_val = version;
+  return retc;
+}
+
+tree_cell *
+nasl_socket_get_ssl_ciphersuite (lex_ctxt * lexic)
+{
+  int soc, result;
+  tree_cell *retc;
+
+  soc = get_int_local_var_by_name (lexic, "socket", -1);
+  result = socket_get_ssl_ciphersuite (soc);
+  if (result < 0)
+    return NULL;
+  retc = alloc_tree_cell (0, NULL);
+  retc->type = CONST_INT;
+  retc->x.i_val = result;
+  return retc;
+}
+
 /*---------------------------------------------------------------------*/
 
 tree_cell *
@@ -463,7 +671,6 @@ nasl_recv (lex_ctxt * lexic)
   fd_set rd;
   struct timeval tv;
   int new_len = 0;
-  tree_cell *retc;
   int type = -1;
   unsigned int opt_len = sizeof (type);
   int e;
@@ -474,8 +681,7 @@ nasl_recv (lex_ctxt * lexic)
   tv.tv_sec = to;
   tv.tv_usec = 0;
 
-
-  data = emalloc (len);
+  data = g_malloc0 (len);
   if (!fd_is_stream (soc))
     e = getsockopt (soc, SOL_SOCKET, SO_TYPE, &type, &opt_len);
   else
@@ -504,7 +710,7 @@ nasl_recv (lex_ctxt * lexic)
                 {
                   if (!new_len)
                     {
-                      efree (&data);
+                      g_free (data);
                       return NULL;
                     }
                   else
@@ -540,16 +746,16 @@ nasl_recv (lex_ctxt * lexic)
     }
   if (new_len > 0)
     {
-      retc = alloc_tree_cell (0, NULL);
+      tree_cell *retc = alloc_tree_cell (0, NULL);
       retc->type = CONST_DATA;
-      retc->x.str_val = nasl_strndup (data, new_len);
+      retc->x.str_val = g_memdup (data, new_len + 1);
       retc->size = new_len;
-      efree (&data);
+      g_free (data);
       return retc;
     }
   else
     {
-      efree (&data);
+      g_free (data);
       return NULL;
     }
 }
@@ -570,8 +776,8 @@ nasl_recv_line (lex_ctxt * lexic)
 
   if (len == -1 || soc <= 0)
     {
-      nasl_perror (lexic,
-                   "recv_line: missing or undefined parameter length or soc\n");
+      nasl_perror (lexic, "recv_line: missing or undefined parameter"
+                          " length or socket\n");
       return NULL;
     }
 
@@ -585,7 +791,7 @@ nasl_recv_line (lex_ctxt * lexic)
         stream_set_buffer (soc, len + 1);
     }
 
-  data = emalloc (len + 1);
+  data = g_malloc0 (len + 1);
   for (;;)
     {
       int e = read_stream_connection_min (soc, data + n, 1, 1);
@@ -607,7 +813,7 @@ nasl_recv_line (lex_ctxt * lexic)
 
   if (n <= 0)
     {
-      efree (&data);
+      g_free (data);
       return NULL;
     }
 
@@ -618,9 +824,9 @@ nasl_recv_line (lex_ctxt * lexic)
   retc = alloc_tree_cell (0, NULL);
   retc->type = CONST_DATA;
   retc->size = new_len;
-  retc->x.str_val = nasl_strndup (data, new_len);
+  retc->x.str_val = g_memdup (data, new_len + 1);
 
-  efree (&data);
+  g_free (data);
 
   return retc;
 }
@@ -699,13 +905,25 @@ nasl_close_socket (lex_ctxt * lexic)
    * the program embedding the NASL interpreter.  Other functions should probably
    * also check the descriptors.
    *
-   * I also wonder whether the original code (disallowing any file descriptor <= 4) 
+   * I also wonder whether the original code (disallowing any file descriptor <= 4)
    * actually was correct and the real defect is that open_sock_udp actually
    * returned 4.  Under which circumstances does it actually do that?  In my brief
    * tests with the stand-alone nasl interpreter the smallest number it returned
    * was 5.
+   *
+   * Update (Hani) 2014-07-14:
+   * 0, 1, 2: Standard streams.
+   * 3: log fd. (dup2() calls in Scanner src/log.c)
+   * 4: thread socket. (dup2() call in Scanner src/nasl_plugins.c)
+   *    Used by the child to send info to the parent. Otherwise close(4) will
+   *    kill the communication with the parent process.
+   * 5: Generally:
+   *  With GnuTLS 2.x: File descriptor open in libgcrypt.
+   *  With GnuTLS 3.x: File descriptor open in libgnutls.
+   *  However, we shouldn't have knowledge or/and make assumptions about external
+   *  libraries.
    */
-  if (soc < 4)
+  if (soc <= 4)
     {
       nasl_perror (lexic, "close(): invalid argument\n");
       return NULL;
@@ -744,11 +962,9 @@ tree_cell *
 nasl_join_multicast_group (lex_ctxt * lexic)
 {
   char *a;
-  int s, i, j;
+  int i, j;
   struct ip_mreq m;
   tree_cell *retc = NULL;
-  void *p;
-
 
   a = get_str_var_by_num (lexic, 0);
   if (a == NULL)
@@ -777,7 +993,7 @@ nasl_join_multicast_group (lex_ctxt * lexic)
 
   if (i >= jmg_max)
     {
-      s = socket (AF_INET, SOCK_DGRAM, 0);
+      int s = socket (AF_INET, SOCK_DGRAM, 0);
       if (s < 0)
         {
           nasl_perror (lexic, "join_multicast_group: socket: %s\n",
@@ -796,14 +1012,7 @@ nasl_join_multicast_group (lex_ctxt * lexic)
 
       if (j < 0)
         {
-          p = erealloc (jmg_desc, sizeof (*jmg_desc) * (jmg_max + 1));
-          if (p == NULL)
-            {
-              nasl_perror (lexic, "join_multicast_group: realloc failed\n");
-              close (s);
-              return NULL;
-            }
-          jmg_desc = p;
+          jmg_desc = g_realloc (jmg_desc, sizeof (*jmg_desc) * (jmg_max + 1));
           j = jmg_max++;
         }
       jmg_desc[j].s = s;
@@ -848,6 +1057,8 @@ nasl_leave_multicast_group (lex_ctxt * lexic)
   return NULL;
 }
 
+
+/* Fixme: Merge this into nasl_get_sock_info.  */
 tree_cell *
 nasl_get_source_port (lex_ctxt * lexic)
 {
@@ -914,12 +1125,8 @@ nasl_socket_get_error (lex_ctxt * lexic)
       break;
     case EBADF:
     case EPIPE:
-#ifdef ECONNRESET
     case ECONNRESET:
-#endif
-#ifdef ENOTSOCK
     case ENOTSOCK:
-#endif
       retc->x.i_val = NASL_ERR_ECONNRESET;
       break;
 
@@ -929,7 +1136,250 @@ nasl_socket_get_error (lex_ctxt * lexic)
       break;
 
     default:
-      fprintf (stderr, "Unknown error %d %s\n", err, strerror (err));
+      log_legacy_write ("Unknown error %d %s\n", err, strerror (err));
+    }
+
+  return retc;
+}
+
+
+/**
+ * @brief Get info pertaining to a socket.
+ * @naslfn{get_sock_info}
+ *
+ * This function is used to retrieve various information about an
+ * active socket.  It requires the NASL socket number and a string to
+ * select the information to retrieve.
+ *
+ * Supported keywords are:
+ *
+ * - @a dport Return the destination port.  This is an integer.  NOTE:
+ *   Not yet implemented.
+ *
+ * - @a sport Return the source port.  This is an integer.  NOTE: Not
+ *   yet implemented.
+ *
+ * - @a encaps Return the encapsulation of the socket.  Example
+ *   output: "TLScustom".
+ *
+ * - @a tls-proto Return a string with the actual TLS protocol in use.
+ *   n/a" is returned if no SSL/TLS session is active.  Example
+ *   output: "TLSv1".
+ *
+ * - @a tls-kx Return a string describing the key exchange algorithm.
+ *   Example output: "RSA".
+ *
+ * - @a tls-certtype Return the type of the certificate in use by the
+ *   session.  Example output: "X.509"
+ *
+ * - @a tls-cipher Return the cipher algorithm in use by the session;
+ *   Example output: "AES-256-CBC".
+ *
+ * - @a tls-mac Return the message authentication algorithms used by
+ *   the session.  Example output: "SHA1".
+ *
+ * - @a tls-comp Return the compression algorithms in use by the
+ *   session.  Example output: "DEFLATE".
+ *
+ * - @a tls-auth Return the peer's authentication type.  Example
+ *   output: "CERT".
+ *
+ * - @a tls-cert Return the peer's certificates for an SSL or TLS
+ *   connection.  This is an array of binary strings or NULL if no
+ *   certificate is known.
+ *
+ * @nasluparam
+ *
+ * - A NASL socket
+ *
+ * - A string keyword; see above.
+ *
+ * @naslnparam
+ *
+ * - @a asstring If true return a human readable string instead of
+ *   an integer.  Used only with these keywords: encaps.
+ *
+ * @naslret An integer or a string or NULL on error.
+ *
+ * @param[in] lexic  Lexical context of the NASL interpreter.
+ *
+ * @return A tree cell.
+ */
+tree_cell *
+nasl_get_sock_info (lex_ctxt * lexic)
+{
+  int sock;
+  int type;
+  int err;
+  const char *keyword, *s;
+  tree_cell *retc;
+  int as_string;
+  int transport;
+  gnutls_session_t tls_session;
+  char *strval;
+  int intval;
+
+  sock = get_int_var_by_num (lexic, 0, -1);
+  if (sock <= 0)
+    {
+      nasl_perror (lexic, "error: socket %d is not valid\n");
+      return NULL;
+    }
+
+  keyword = get_str_var_by_num (lexic, 1);
+  if (!keyword || !((type = get_var_type_by_num (lexic, 1)) == VAR2_STRING
+                    || type == VAR2_DATA))
+    {
+      nasl_perror (lexic, "error: second argument is not of type string\n");
+      return NULL;
+    }
+
+  as_string = !!get_int_local_var_by_name (lexic, "asstring", 0);
+
+  transport = 0;
+  strval = NULL;
+  intval = 0;
+  retc = FAKE_CELL; /* Dummy value to detect retc == NULL.  */
+
+  {
+    void *tmp = NULL;
+    err = get_sock_infos (sock, &transport, &tmp);
+    tls_session = tmp;
+  }
+  if (err)
+    {
+      nasl_perror (lexic, "error retrieving infos for socket %d: %s\n",
+                   sock, strerror (err));
+      retc = NULL;
+    }
+  else if (!strcmp (keyword, "encaps"))
+    {
+      if (as_string)
+        strval = g_strdup (get_encaps_name (transport));
+      else
+        intval = transport;
+    }
+  else if (!strcmp (keyword, "tls-proto"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_protocol_get_name
+          (gnutls_protocol_get_version (tls_session));
+      strval = g_strdup (s?s:"[?]");
+    }
+  else if (!strcmp (keyword, "tls-kx"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_kx_get_name (gnutls_kx_get (tls_session));
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-certtype"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_certificate_type_get_name
+          (gnutls_certificate_type_get (tls_session));
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-cipher"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_cipher_get_name (gnutls_cipher_get (tls_session));
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-mac"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_mac_get_name (gnutls_mac_get (tls_session));
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-comp"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        s = gnutls_compression_get_name
+          (gnutls_compression_get (tls_session));
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-auth"))
+    {
+      if (!tls_session)
+        s = "n/a";
+      else
+        {
+          switch (gnutls_auth_get_type (tls_session))
+            {
+            case GNUTLS_CRD_ANON:        s = "ANON"; break;
+            case GNUTLS_CRD_CERTIFICATE: s = "CERT"; break;
+            case GNUTLS_CRD_PSK:         s = "PSK";  break;
+            case GNUTLS_CRD_SRP:         s = "SRP";  break;
+            default:                     s = "[?]";  break;
+            }
+        }
+      strval = g_strdup (s?s:"");
+    }
+  else if (!strcmp (keyword, "tls-cert"))
+    {
+      /* We only support X.509 for now.  GNUTLS also allows for
+         OpenPGP, but we are not prepared for that.  */
+      if (tls_session
+          && gnutls_certificate_type_get (tls_session) == GNUTLS_CRT_X509)
+        {
+          const gnutls_datum_t *list;
+          unsigned int nlist = 0;
+          nasl_array *a;
+          anon_nasl_var v;
+
+          list = gnutls_certificate_get_peers (tls_session, &nlist);
+          if (!list)
+            retc = NULL;  /* No certificate or other error.  */
+          else
+            {
+              int i;
+              retc = alloc_tree_cell (0, NULL);
+              retc->type = DYN_ARRAY;
+              retc->x.ref_val = a = g_malloc0 (sizeof *a);
+
+              for (i=0; i < nlist; i++)
+                {
+                  memset (&v, 0, sizeof v);
+                  v.var_type = VAR2_DATA;
+                  v.v.v_str.s_val = list[i].data;
+                  v.v.v_str.s_siz = list[i].size;
+                  add_var_to_list (a, i, &v);
+                }
+            }
+        }
+    }
+  else
+    {
+      nasl_perror (lexic, "unknown keyword '%s'\n", keyword);
+      retc = NULL;
+    }
+
+  if (!retc)
+    ;
+  else if (retc != FAKE_CELL)
+    ; /* Already allocated.  */
+  else if (strval)
+    {
+      retc = alloc_typed_cell (CONST_STR);
+      retc->x.str_val = strval;
+      retc->size = strlen (strval);
+    }
+  else
+    {
+      retc = alloc_typed_cell (CONST_INT);
+      retc->x.i_val = intval;
     }
 
   return retc;
