@@ -1,4 +1,4 @@
-/* Nessus Attack Scripting Language 
+/* Nessus Attack Scripting Language
  *
  * Copyright (C) 2002 - 2004 Tenable Network Security
  *
@@ -13,20 +13,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
 #include <stdlib.h>             /* for abort */
 #include <string.h>             /* for strlen */
 
+#include <glib.h>               /* for g_free */
+
 #include "nasl_tree.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_func.h"
 #include "nasl_var.h"
 #include "nasl_lex_ctxt.h"
 #include "exec.h"
-#include "system.h"
 #include "nasl_debug.h"
 
 #ifndef NASL_DEBUG
@@ -85,12 +86,12 @@ nasl_get_var_by_num (nasl_array * a, int num, int create)
 
   if (num >= a->max_idx)
     {
-      a->num_elt = erealloc (a->num_elt, sizeof (anon_nasl_var *) * (num + 1));
+      a->num_elt = g_realloc (a->num_elt, sizeof (anon_nasl_var *) * (num + 1));
       bzero (a->num_elt + a->max_idx,
              sizeof (anon_nasl_var *) * (num + 1 - a->max_idx));
       a->max_idx = num + 1;
     }
-  v = emalloc (sizeof (anon_nasl_var));
+  v = g_malloc0 (sizeof (anon_nasl_var));
   v->var_type = VAR2_UNDEF;
 
   a->num_elt[num] = v;
@@ -104,14 +105,14 @@ get_var_by_name (nasl_array * a, const char *s)
   named_nasl_var *v;
 
   if (a->hash_elt == NULL)
-    a->hash_elt = emalloc (VAR_NAME_HASH * sizeof (named_nasl_var *));
+    a->hash_elt = g_malloc0 (VAR_NAME_HASH * sizeof (named_nasl_var *));
 
   for (v = a->hash_elt[h]; v != NULL; v = v->next_var)
     if (v->var_name != NULL && strcmp (s, v->var_name) == 0)
       return v;
 
-  v = emalloc (sizeof (named_nasl_var));
-  v->var_name = estrdup (s);
+  v = g_malloc0 (sizeof (named_nasl_var));
+  v->var_name = g_strdup (s);
   v->u.var_type = VAR2_UNDEF;
   v->next_var = a->hash_elt[h];
 
@@ -126,7 +127,7 @@ get_var_by_name (nasl_array * a, const char *s)
 static named_nasl_var *
 get_var_ref_by_name (lex_ctxt * ctxt, const char *name, int climb)
 {
-  named_nasl_var *v, *prev;
+  named_nasl_var *v;
   int h = hash_str (name);
   lex_ctxt *c;
 
@@ -134,54 +135,24 @@ get_var_ref_by_name (lex_ctxt * ctxt, const char *name, int climb)
     {
       for (c = ctxt; c != NULL; c = c->up_ctxt)
         if (c->ctx_vars.hash_elt != NULL)
-          for (prev = NULL, v = c->ctx_vars.hash_elt[h]; v != NULL;
-               v = v->next_var)
+          for (v = c->ctx_vars.hash_elt[h]; v != NULL; v = v->next_var)
             if (v->var_name != NULL && strcmp (name, v->var_name) == 0)
-              {
-#ifdef SILLY_OPT
-                if (prev != NULL)       /* Move it to start of list */
-                  {
-                    prev->next_var = v->next_var;
-                    v->next_var = c->ctx_vars.hash_elt[h];
-                    c->ctx_vars.hash_elt[h] = v;
-                  }
-#endif
-                return v;
-              }
-#ifdef SILLY_OPT
-            else
-              prev = v;
-#endif
+              return v;
     }
   else
     {
       if (ctxt->ctx_vars.hash_elt != NULL)
-        for (prev = NULL, v = ctxt->ctx_vars.hash_elt[h]; v != NULL;
-             v = v->next_var)
+        for (v = ctxt->ctx_vars.hash_elt[h]; v != NULL; v = v->next_var)
           if (v->var_name != NULL && strcmp (name, v->var_name) == 0)
-            {
-#ifdef SILLY_OPT
-              if (prev != NULL) /* Move it to start of list */
-                {
-                  prev->next_var = v->next_var;
-                  v->next_var = ctxt->ctx_vars.hash_elt[h];
-                  ctxt->ctx_vars.hash_elt[h] = v;
-                }
-#endif
-              return v;
-            }
-#ifdef SILLY_OPT
-          else
-            prev = v;
-#endif
+            return v;
     }
 
   if (ctxt->ctx_vars.hash_elt == NULL)
     ctxt->ctx_vars.hash_elt =
-      emalloc (sizeof (named_nasl_var *) * VAR_NAME_HASH);
+      g_malloc0 (sizeof (named_nasl_var *) * VAR_NAME_HASH);
 
-  v = emalloc (sizeof (named_nasl_var));
-  v->var_name = estrdup (name);
+  v = g_malloc0 (sizeof (named_nasl_var));
+  v->var_name = g_strdup (name);
   v->u.var_type = VAR2_UNDEF;
   v->next_var = ctxt->ctx_vars.hash_elt[h];
   ctxt->ctx_vars.hash_elt[h] = v;
@@ -204,7 +175,7 @@ get_var_ref_by_num (lex_ctxt * ctxt, int num)
   if (ctxt->ctx_vars.max_idx <= num)
     {
       ctxt->ctx_vars.num_elt =
-        erealloc (ctxt->ctx_vars.num_elt, sizeof (anon_nasl_var *) * (num + 1));
+        g_realloc (ctxt->ctx_vars.num_elt, sizeof (anon_nasl_var *) * (num + 1));
       bzero (ctxt->ctx_vars.num_elt + ctxt->ctx_vars.max_idx,
              sizeof (anon_nasl_var *) * (num + 1 - ctxt->ctx_vars.max_idx));
       ctxt->ctx_vars.max_idx = num + 1;
@@ -214,7 +185,7 @@ get_var_ref_by_num (lex_ctxt * ctxt, int num)
   if (v != NULL)
     return v;
 
-  v = emalloc (sizeof (anon_nasl_var));
+  v = g_malloc0 (sizeof (anon_nasl_var));
   v->var_type = VAR2_UNDEF;
   ctxt->ctx_vars.num_elt[num] = v;
   return v;
@@ -239,7 +210,7 @@ get_variable_by_name (lex_ctxt * ctxt, const char *name)
   if (strcmp (name, "_FCT_ANON_ARGS") == 0)
     {
       tree_cell *retc = alloc_typed_cell (DYN_ARRAY);
-      nasl_array *a = retc->x.ref_val = emalloc (sizeof (nasl_array));
+      nasl_array *a = retc->x.ref_val = g_malloc0 (sizeof (nasl_array));
       copy_array (a, &ctxt->ctx_vars, 0);
       return retc;
     }
@@ -343,7 +314,7 @@ get_array_elem (lex_ctxt * ctxt, const char *name, tree_cell * idx)
                            name, idx->x.i_val, l);
               tc = alloc_expr_cell (idx->line_nb, CONST_DATA /*CONST_STR */ ,
                                     NULL, NULL);
-              tc->x.str_val = estrdup ("");
+              tc->x.str_val = g_strdup ("");
               tc->size = 0;
               return tc;
             }
@@ -356,7 +327,7 @@ get_array_elem (lex_ctxt * ctxt, const char *name, tree_cell * idx)
                 }
               tc = alloc_expr_cell (idx->line_nb, CONST_DATA /*CONST_STR */ ,
                                     NULL, NULL);
-              tc->x.str_val = emalloc (2);
+              tc->x.str_val = g_malloc0 (2);
               tc->x.str_val[0] = u->v.v_str.s_val[idx->x.i_val];
               tc->x.str_val[1] = '\0';
               tc->size = 1;
@@ -398,15 +369,17 @@ free_array (nasl_array * a)
     {
       for (i = 0; i < a->max_idx; i++)
         free_anon_var (a->num_elt[i]);
-      efree (&a->num_elt);
+      g_free (a->num_elt);
+      a->num_elt = NULL;
     }
+  a->max_idx = 0;
   if (a->hash_elt != NULL)
     {
       for (i = 0; i < VAR_NAME_HASH; i++)
         free_var_chain (a->hash_elt[i]);
-      efree (&a->hash_elt);
+      g_free (a->hash_elt);
+      a->num_elt = NULL;
     }
-  return;
 }
 
 static void
@@ -416,18 +389,18 @@ free_var_chain (named_nasl_var * v)
   if (v == NULL)
     return;
   free_var_chain (v->next_var);
-  efree (&v->var_name);
+  g_free (v->var_name);
   switch (v->u.var_type)
     {
     case VAR2_STRING:
     case VAR2_DATA:
-      efree (&v->u.v.v_str.s_val);
+      g_free (v->u.v.v_str.s_val);
       break;
     case VAR2_ARRAY:
       free_array (&v->u.v.v_arr);
       break;
     }
-  efree (&v);
+  g_free (v);
 }
 
 static void
@@ -439,34 +412,13 @@ free_anon_var (anon_nasl_var * v)
     {
     case VAR2_STRING:
     case VAR2_DATA:
-      efree (&v->v.v_str.s_val);
+      g_free (v->v.v_str.s_val);
       break;
     case VAR2_ARRAY:
       free_array (&v->v.v_arr);
       break;
     }
-  efree (&v);
-
-}
-
-static void
-clear_array (nasl_array * a)
-{
-  int i;
-
-  if (a->num_elt != NULL)
-    {
-      for (i = 0; i < a->max_idx; i++)
-        free_anon_var (a->num_elt[i]);
-      efree (&a->num_elt);
-    }
-  a->max_idx = 0;
-  if (a->hash_elt != NULL)
-    {
-      for (i = 0; i < VAR_NAME_HASH; i++)
-        free_var_chain (a->hash_elt[i]);
-      efree (&a->hash_elt);
-    }
+  g_free (v);
 }
 
 void
@@ -483,15 +435,15 @@ clear_anon_var (anon_nasl_var * v)
       break;
     case VAR2_STRING:
     case VAR2_DATA:
-      efree (&v->v.v_str.s_val);
+      g_free (v->v.v_str.s_val);
+      v->v.v_str.s_val = NULL;
       v->v.v_str.s_siz = 0;
       break;
     case VAR2_ARRAY:
-      clear_array (&v->v.v_arr);
+      free_array (&v->v.v_arr);
       break;
     }
   v->var_type = VAR2_UNDEF;
-  return;
 }
 
 static void
@@ -505,7 +457,7 @@ copy_anon_var (anon_nasl_var * v1, const anon_nasl_var * v2)
     case VAR2_DATA:
       if (v2->v.v_str.s_val != NULL)
         {
-          v1->v.v_str.s_val = emalloc (v2->v.v_str.s_siz);
+          v1->v.v_str.s_val = g_malloc0 (v2->v.v_str.s_siz + 1);
           memcpy (v1->v.v_str.s_val, v2->v.v_str.s_val, v2->v.v_str.s_siz);
           v1->v.v_str.s_siz = v2->v.v_str.s_siz;
         }
@@ -541,7 +493,7 @@ dup_anon_var (const anon_nasl_var * v)
   if (v == NULL)
     return NULL;
 
-  v1 = emalloc (sizeof (anon_nasl_var));
+  v1 = g_malloc0 (sizeof (anon_nasl_var));
   copy_anon_var (v1, v);
   return v1;
 }
@@ -554,9 +506,9 @@ dup_named_var (const named_nasl_var * v)
   if (v == NULL)
     return NULL;
 
-  v1 = emalloc (sizeof (named_nasl_var));
+  v1 = g_malloc0 (sizeof (named_nasl_var));
   copy_anon_var (&v1->u, &v->u);
-  v1->var_name = estrdup (v->var_name);
+  v1->var_name = g_strdup (v->var_name);
   return v1;
 }
 
@@ -580,18 +532,18 @@ copy_array (nasl_array * a1, const nasl_array * a2, int copy_named)
       abort ();
     }
 
-  clear_array (a1);
+  free_array (a1);
 
   if (a2->num_elt != NULL)
     {
       a1->max_idx = a2->max_idx;
-      a1->num_elt = emalloc (sizeof (anon_nasl_var *) * a2->max_idx);
+      a1->num_elt = g_malloc0 (sizeof (anon_nasl_var *) * a2->max_idx);
       for (i = 0; i < a2->max_idx; i++)
         a1->num_elt[i] = dup_anon_var (a2->num_elt[i]);
     }
   if (copy_named && a2->hash_elt != NULL)
     {
-      a1->hash_elt = emalloc (VAR_NAME_HASH * sizeof (named_nasl_var *));
+      a1->hash_elt = g_malloc0 (VAR_NAME_HASH * sizeof (named_nasl_var *));
       for (i = 0; i < VAR_NAME_HASH; i++)
         {
           v1 = NULL;
@@ -617,7 +569,7 @@ copy_ref_array (const tree_cell * c1)
 
   c2 = alloc_tree_cell (0, NULL);
   c2->type = DYN_ARRAY;
-  c2->x.ref_val = a2 = emalloc (sizeof (nasl_array));
+  c2->x.ref_val = a2 = g_malloc0 (sizeof (nasl_array));
   copy_array (a2, c1->x.ref_val, 1);
   return c2;
 }
@@ -708,7 +660,7 @@ affect_to_anon_var (anon_nasl_var * v1, tree_cell * rval)
     }
 
   /*
-   * Bug #146: when executing 
+   * Bug #146: when executing
    *    x = 'abc'; x = x;  or   x = make_list(...); x = x[0];
    * the rvalue will be freed before it is copied to the lvalue
    */
@@ -760,8 +712,8 @@ affect_to_anon_var (anon_nasl_var * v1, tree_cell * rval)
           }
         else
           {
-            p = emalloc (rval->size + 1);
-            memcpy (p, rval->x.str_val, rval->size + 1);
+            p = g_malloc0 (rval->size + 1);
+            memcpy (p, rval->x.str_val, rval->size);
             v1->v.v_str.s_siz = rval->size;
             v1->v.v_str.s_val = p;
           }
@@ -782,7 +734,7 @@ affect_to_anon_var (anon_nasl_var * v1, tree_cell * rval)
           }
         else
           {
-            p = emalloc (v2->v.v_str.s_siz);
+            p = g_malloc0 (v2->v.v_str.s_siz + 1);
             memcpy (p, v2->v.v_str.s_val, v2->v.v_str.s_siz);
             v1->v.v_str.s_siz = v2->v.v_str.s_siz;
             v1->v.v_str.s_val = p;
@@ -847,11 +799,11 @@ nasl_affect (tree_cell * lval, tree_cell * rval)
 static named_nasl_var *
 create_named_var (const char *name, tree_cell * val)
 {
-  named_nasl_var *v = emalloc (sizeof (named_nasl_var));
+  named_nasl_var *v = g_malloc0 (sizeof (named_nasl_var));
   tree_cell *tc;
 
   if (name != NULL)
-    v->var_name = estrdup (name);
+    v->var_name = g_strdup (name);
 
   if (val == NULL || val == FAKE_CELL)
     {
@@ -873,7 +825,7 @@ create_named_var (const char *name, tree_cell * val)
 static anon_nasl_var *
 create_anon_var (tree_cell * val)
 {
-  anon_nasl_var *v = emalloc (sizeof (anon_nasl_var));
+  anon_nasl_var *v = g_malloc0 (sizeof (anon_nasl_var));
   tree_cell *tc;
 
   if (val == NULL || val == FAKE_CELL)
@@ -939,7 +891,7 @@ add_numbered_var_to_ctxt (lex_ctxt * lexic, int num, tree_cell * val)
     }
   else
     {
-      a->num_elt = erealloc (a->num_elt, (num + 1) * sizeof (anon_nasl_var));
+      a->num_elt = g_realloc (a->num_elt, (num + 1) * sizeof (anon_nasl_var));
       bzero (a->num_elt + a->max_idx,
              sizeof (anon_nasl_var *) * (num + 1 - a->max_idx));
       a->max_idx = num + 1;
@@ -1035,7 +987,7 @@ nasl_read_var_ref (lex_ctxt * lexic, tree_cell * tc)
         }
       else
         {
-          ret->x.str_val = emalloc (v->v.v_str.s_siz);
+          ret->x.str_val = g_malloc0 (v->v.v_str.s_siz + 1);
           memcpy (ret->x.str_val, v->v.v_str.s_val, v->v.v_str.s_siz);
           ret->size = v->v.v_str.s_siz;
         }
@@ -1176,7 +1128,7 @@ array2str (const nasl_array * a)
   if (len == 0)
     {
       len = 80;
-      s = emalloc (80);
+      s = g_malloc0 (80 + 1);
     }
 
   strcpy (s, "[ ");
@@ -1189,7 +1141,7 @@ array2str (const nasl_array * a)
           if (n + 80 >= len)
             {
               len += 80;
-              s = erealloc (s, len);
+              s = g_realloc (s, len + 1);
             }
           if (n1 > 0)
             n += sprintf (s + n, ", ");
@@ -1231,7 +1183,7 @@ array2str (const nasl_array * a)
             if (n + 80 >= len)
               {
                 len += 80 + l;
-                s = erealloc (s, len);
+                s = g_realloc (s, len + 1);
               }
             if (n1 > 0)
               n += sprintf (s + n, ", ");
@@ -1265,7 +1217,7 @@ array2str (const nasl_array * a)
   if (n + 2 >= len)
     {
       len += 80;
-      s = erealloc (s, len);
+      s = g_realloc (s, len + 1);
     }
   strcpy (s + n, " ]");
   return s;
@@ -1404,13 +1356,6 @@ get_var_type_by_num (lex_ctxt * lexic, int num)
   return v == NULL ? VAR2_UNDEF : v->var_type;
 }
 
-int
-get_var_type_by_name (lex_ctxt * lexic, const char *name)
-{
-  named_nasl_var *v = get_var_ref_by_name (lexic, name, 1);
-  return v == NULL ? VAR2_UNDEF : v->u.var_type;
-}
-
 int
 get_local_var_type_by_name (lex_ctxt * lexic, const char *name)
 {
@@ -1506,7 +1451,7 @@ add_var_to_list (nasl_array * a, int i, const anon_nasl_var * v)
 
   if (i >= a->max_idx)
     {
-      a->num_elt = erealloc (a->num_elt, sizeof (anon_nasl_var *) * (i + 1));
+      a->num_elt = g_realloc (a->num_elt, sizeof (anon_nasl_var *) * (i + 1));
       bzero (a->num_elt + a->max_idx,
              sizeof (anon_nasl_var *) * (i + 1 - a->max_idx));
       a->max_idx = i + 1;
@@ -1529,11 +1474,11 @@ add_var_to_array (nasl_array * a, char *name, const anon_nasl_var * v)
 
   if (a->hash_elt == NULL)
     {
-      a->hash_elt = emalloc (VAR_NAME_HASH * sizeof (named_nasl_var *));
+      a->hash_elt = g_malloc0 (VAR_NAME_HASH * sizeof (named_nasl_var *));
     }
 
-  v2 = emalloc (sizeof (named_nasl_var));
-  v2->var_name = estrdup (name);
+  v2 = g_malloc0 (sizeof (named_nasl_var));
+  v2->var_name = g_strdup (name);
   v2->u.var_type = VAR2_UNDEF;
   v2->next_var = a->hash_elt[h];
   a->hash_elt[h] = v2;
@@ -1575,20 +1520,20 @@ make_array_from_elems (tree_cell * el)
   nasl_array *a;
   anon_nasl_var v;
 
-  a = emalloc (sizeof (nasl_array));
+  a = g_malloc0 (sizeof (nasl_array));
   /* Either the elements are all "named", or they are "numbered". No mix! */
   if (el->x.str_val == NULL)    /* numbered */
     {
       for (n = 0, c = el; c != NULL; c = c->link[1])
         n++;
       a->max_idx = n;
-      a->num_elt = emalloc (sizeof (anon_nasl_var *) * n);
+      a->num_elt = g_malloc0 (sizeof (anon_nasl_var *) * n);
       a->hash_elt = NULL;
     }
   else
     {
       a->num_elt = NULL;
-      a->hash_elt = emalloc (VAR_NAME_HASH * sizeof (named_nasl_var *));
+      a->hash_elt = g_malloc0 (VAR_NAME_HASH * sizeof (named_nasl_var *));
     }
 
   for (n = 0, c = el; c != NULL; c = c->link[1])
